=begin
#NSX-T Manager API

#VMware NSX-T Manager REST API

OpenAPI spec version: 2.5.1.0.0

Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 2.4.19

=end

require 'date'

module NSXT
  # Information of a network interface present on the partner appliance that needs to be configured by the NSX Manager.
  class NicInfo
    # Subnet mask associated with the NIC metadata.
    attr_accessor :subnet_mask

    # Gateway address associated with the NIC metadata.
    attr_accessor :gateway_address

    # IP allocation type with values STATIC, DHCP, or NONE indicating that IP address is not required.
    attr_accessor :ip_allocation_type

    # NIC metadata information.
    attr_accessor :nic_metadata

    # Network Id associated with the NIC metadata. It can be a moref, or a logical switch ID. If it is to be taken from 'Agent VM Settings', then it should be empty.
    attr_accessor :network_id

    # If the nic should get IP using a static IP pool then IP pool id should be provided here.
    attr_accessor :ip_pool_id

    # IP address associated with the NIC metadata. Required only when assigning IP statically for a deployment that is for a single VM instance.
    attr_accessor :ip_address

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'subnet_mask' => :'subnet_mask',
        :'gateway_address' => :'gateway_address',
        :'ip_allocation_type' => :'ip_allocation_type',
        :'nic_metadata' => :'nic_metadata',
        :'network_id' => :'network_id',
        :'ip_pool_id' => :'ip_pool_id',
        :'ip_address' => :'ip_address'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'subnet_mask' => :'String',
        :'gateway_address' => :'String',
        :'ip_allocation_type' => :'String',
        :'nic_metadata' => :'NicMetadata',
        :'network_id' => :'String',
        :'ip_pool_id' => :'String',
        :'ip_address' => :'String'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'subnet_mask')
        self.subnet_mask = attributes[:'subnet_mask']
      end

      if attributes.has_key?(:'gateway_address')
        self.gateway_address = attributes[:'gateway_address']
      end

      if attributes.has_key?(:'ip_allocation_type')
        self.ip_allocation_type = attributes[:'ip_allocation_type']
      end

      if attributes.has_key?(:'nic_metadata')
        self.nic_metadata = attributes[:'nic_metadata']
      end

      if attributes.has_key?(:'network_id')
        self.network_id = attributes[:'network_id']
      end

      if attributes.has_key?(:'ip_pool_id')
        self.ip_pool_id = attributes[:'ip_pool_id']
      end

      if attributes.has_key?(:'ip_address')
        self.ip_address = attributes[:'ip_address']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      ip_allocation_type_validator = EnumAttributeValidator.new('String', ['STATIC', 'DHCP', 'NONE'])
      return false unless ip_allocation_type_validator.valid?(@ip_allocation_type)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] ip_allocation_type Object to be assigned
    def ip_allocation_type=(ip_allocation_type)
      validator = EnumAttributeValidator.new('String', ['STATIC', 'DHCP', 'NONE'])
      unless validator.valid?(ip_allocation_type)
        fail ArgumentError, 'invalid value for "ip_allocation_type", must be one of #{validator.allowable_values}.'
      end
      @ip_allocation_type = ip_allocation_type
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          subnet_mask == o.subnet_mask &&
          gateway_address == o.gateway_address &&
          ip_allocation_type == o.ip_allocation_type &&
          nic_metadata == o.nic_metadata &&
          network_id == o.network_id &&
          ip_pool_id == o.ip_pool_id &&
          ip_address == o.ip_address
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [subnet_mask, gateway_address, ip_allocation_type, nic_metadata, network_id, ip_pool_id, ip_address].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = NSXT.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
