=begin
#NSX-T Data Center Policy API

#VMware NSX-T Data Center Policy REST API

OpenAPI spec version: 3.1.0.0.0

Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 2.4.19

=end

require 'date'

module NSXTPolicy
  # Intersite gateway configuration.
  class IntersiteGatewayConfig
    # IPv4 subnet for inter-site transit segment connecting service routers across sites for stretched gateway. For IPv6 link local subnet is auto configured. 
    attr_accessor :intersite_transit_subnet

    # Primary egress site for gateway. T0/T1 gateway in Active/Standby mode supports stateful services on primary site. In this mode primary site must be set if gateway is stretched to more than one site. For T0 gateway in Active/Active primary site is optional field. If set then secondary site prefers routes learned from primary over locally learned routes. This field is not applicable for T1 gateway with no services. 
    attr_accessor :primary_site_path

    # Epoch(in seconds) is auto updated based on system current timestamp when primary locale service is updated. It is used for resolving conflict during site failover. If system clock not in sync then User can optionally override this. New value must be higher than the current value. 
    attr_accessor :last_admin_active_epoch

    # Fallback site to be used as new primary site on current primary site failure. Disaster recovery must be initiated via API/UI. Fallback site configuration is supported only for T0 gateway. T1 gateway will follow T0 gateway's primary site during disaster recovery. 
    attr_accessor :fallback_sites

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'intersite_transit_subnet' => :'intersite_transit_subnet',
        :'primary_site_path' => :'primary_site_path',
        :'last_admin_active_epoch' => :'last_admin_active_epoch',
        :'fallback_sites' => :'fallback_sites'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'intersite_transit_subnet' => :'String',
        :'primary_site_path' => :'String',
        :'last_admin_active_epoch' => :'Integer',
        :'fallback_sites' => :'Array<String>'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'intersite_transit_subnet')
        self.intersite_transit_subnet = attributes[:'intersite_transit_subnet']
      else
        self.intersite_transit_subnet = '169.254.32.0/20'
      end

      if attributes.has_key?(:'primary_site_path')
        self.primary_site_path = attributes[:'primary_site_path']
      end

      if attributes.has_key?(:'last_admin_active_epoch')
        self.last_admin_active_epoch = attributes[:'last_admin_active_epoch']
      end

      if attributes.has_key?(:'fallback_sites')
        if (value = attributes[:'fallback_sites']).is_a?(Array)
          self.fallback_sites = value
        end
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if !@last_admin_active_epoch.nil? && @last_admin_active_epoch > 4294967295
        invalid_properties.push('invalid value for "last_admin_active_epoch", must be smaller than or equal to 4294967295.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if !@last_admin_active_epoch.nil? && @last_admin_active_epoch > 4294967295
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] last_admin_active_epoch Value to be assigned
    def last_admin_active_epoch=(last_admin_active_epoch)
      if !last_admin_active_epoch.nil? && last_admin_active_epoch > 4294967295
        fail ArgumentError, 'invalid value for "last_admin_active_epoch", must be smaller than or equal to 4294967295.'
      end

      @last_admin_active_epoch = last_admin_active_epoch
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          intersite_transit_subnet == o.intersite_transit_subnet &&
          primary_site_path == o.primary_site_path &&
          last_admin_active_epoch == o.last_admin_active_epoch &&
          fallback_sites == o.fallback_sites
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [intersite_transit_subnet, primary_site_path, last_admin_active_epoch, fallback_sites].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = NSXTPolicy.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
