=begin
#NSX-T Data Center Policy API

#VMware NSX-T Data Center Policy REST API

OpenAPI spec version: 3.1.0.0.0

Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 2.4.19

=end

require 'date'

module NSXTPolicy
  # The load balancer node usage summary for all nodes from specific enforcement point. Only edge node is supported. The summary contains all edge nodes which are configured in edge clusters. 
  class LBNodeUsageSummary
    # Alarm information details.
    attr_accessor :alarm

    # Policy Path referencing the enforcement point where the info is fetched. 
    attr_accessor :enforcement_point_path

    # Current load balancer credits in use for all nodes. For example, configuring a medium load balancer on a node consumes 10 credits. If there are 2 medium instances configured, the current load balancer credit number is 2 * 10 = 20. 
    attr_accessor :current_load_balancer_credits

    # The load balancer credit capacity means the maximum credits which can be used for load balancer service configuration for all nodes. 
    attr_accessor :load_balancer_credit_capacity

    # The property identifies all LB node usages. By default, it is not included in response. It exists when parameter ?include_usages=true. 
    attr_accessor :node_usages

    # The severity calculation is based on overall credit usage percentage of load balancer for all nodes. 
    attr_accessor :severity

    # Pool member capacity means maximum number of pool members which can be configured on all nodes. 
    attr_accessor :pool_member_capacity

    # The property identifies array of node count for each severity. 
    attr_accessor :node_counts

    # The overall count of pool members configured on all nodes. 
    attr_accessor :current_pool_member_count

    # The overall usage percentage of all nodes for load balancer. The value is the larger value between overall pool member usage percentage and overall load balancer credit usage percentage. 
    attr_accessor :usage_percentage

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'alarm' => :'alarm',
        :'enforcement_point_path' => :'enforcement_point_path',
        :'current_load_balancer_credits' => :'current_load_balancer_credits',
        :'load_balancer_credit_capacity' => :'load_balancer_credit_capacity',
        :'node_usages' => :'node_usages',
        :'severity' => :'severity',
        :'pool_member_capacity' => :'pool_member_capacity',
        :'node_counts' => :'node_counts',
        :'current_pool_member_count' => :'current_pool_member_count',
        :'usage_percentage' => :'usage_percentage'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'alarm' => :'PolicyRuntimeAlarm',
        :'enforcement_point_path' => :'String',
        :'current_load_balancer_credits' => :'Integer',
        :'load_balancer_credit_capacity' => :'Integer',
        :'node_usages' => :'Array<LBNodeUsage>',
        :'severity' => :'String',
        :'pool_member_capacity' => :'Integer',
        :'node_counts' => :'Array<LBNodeCountPerSeverity>',
        :'current_pool_member_count' => :'Integer',
        :'usage_percentage' => :'Float'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'alarm')
        self.alarm = attributes[:'alarm']
      end

      if attributes.has_key?(:'enforcement_point_path')
        self.enforcement_point_path = attributes[:'enforcement_point_path']
      end

      if attributes.has_key?(:'current_load_balancer_credits')
        self.current_load_balancer_credits = attributes[:'current_load_balancer_credits']
      end

      if attributes.has_key?(:'load_balancer_credit_capacity')
        self.load_balancer_credit_capacity = attributes[:'load_balancer_credit_capacity']
      end

      if attributes.has_key?(:'node_usages')
        if (value = attributes[:'node_usages']).is_a?(Array)
          self.node_usages = value
        end
      end

      if attributes.has_key?(:'severity')
        self.severity = attributes[:'severity']
      end

      if attributes.has_key?(:'pool_member_capacity')
        self.pool_member_capacity = attributes[:'pool_member_capacity']
      end

      if attributes.has_key?(:'node_counts')
        if (value = attributes[:'node_counts']).is_a?(Array)
          self.node_counts = value
        end
      end

      if attributes.has_key?(:'current_pool_member_count')
        self.current_pool_member_count = attributes[:'current_pool_member_count']
      end

      if attributes.has_key?(:'usage_percentage')
        self.usage_percentage = attributes[:'usage_percentage']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      severity_validator = EnumAttributeValidator.new('String', ['GREEN', 'ORANGE', 'RED'])
      return false unless severity_validator.valid?(@severity)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] severity Object to be assigned
    def severity=(severity)
      validator = EnumAttributeValidator.new('String', ['GREEN', 'ORANGE', 'RED'])
      unless validator.valid?(severity)
        fail ArgumentError, 'invalid value for "severity", must be one of #{validator.allowable_values}.'
      end
      @severity = severity
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          alarm == o.alarm &&
          enforcement_point_path == o.enforcement_point_path &&
          current_load_balancer_credits == o.current_load_balancer_credits &&
          load_balancer_credit_capacity == o.load_balancer_credit_capacity &&
          node_usages == o.node_usages &&
          severity == o.severity &&
          pool_member_capacity == o.pool_member_capacity &&
          node_counts == o.node_counts &&
          current_pool_member_count == o.current_pool_member_count &&
          usage_percentage == o.usage_percentage
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [alarm, enforcement_point_path, current_load_balancer_credits, load_balancer_credit_capacity, node_usages, severity, pool_member_capacity, node_counts, current_pool_member_count, usage_percentage].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = NSXTPolicy.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
