=begin
#NSX-T Data Center Policy API

#VMware NSX-T Data Center Policy REST API

OpenAPI spec version: 3.1.0.0.0

Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 2.4.19

=end

require 'date'

module NSXTPolicy
  # The current statistics counters of the DNS forwarder including cache usages and query numbers per forwarders, on an NSX-T type of enforcement point. 
  class NsxTDNSForwarderStatistics
    attr_accessor :resource_type

    # Policy path referencing the enforcement point from where the statistics are fetched. 
    attr_accessor :enforcement_point_path

    # The total number of forwarded DNS queries
    attr_accessor :queries_forwarded

    # The total number of cached entries
    attr_accessor :cached_entries

    # The statistics of default forwarder zone
    attr_accessor :default_forwarder_statistics

    # The total number of queries answered from local cache
    attr_accessor :queries_answered_locally

    # The statistics of used cache
    attr_accessor :used_cache_statistics

    # The configured cache size, in kb
    attr_accessor :configured_cache_size

    # Time stamp of the current statistics, in ms
    attr_accessor :timestamp

    # The statistics of conditional forwarder zones
    attr_accessor :conditional_forwarder_statistics

    # The total number of received DNS queries
    attr_accessor :total_queries

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'resource_type' => :'resource_type',
        :'enforcement_point_path' => :'enforcement_point_path',
        :'queries_forwarded' => :'queries_forwarded',
        :'cached_entries' => :'cached_entries',
        :'default_forwarder_statistics' => :'default_forwarder_statistics',
        :'queries_answered_locally' => :'queries_answered_locally',
        :'used_cache_statistics' => :'used_cache_statistics',
        :'configured_cache_size' => :'configured_cache_size',
        :'timestamp' => :'timestamp',
        :'conditional_forwarder_statistics' => :'conditional_forwarder_statistics',
        :'total_queries' => :'total_queries'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'resource_type' => :'String',
        :'enforcement_point_path' => :'String',
        :'queries_forwarded' => :'Integer',
        :'cached_entries' => :'Integer',
        :'default_forwarder_statistics' => :'NsxTDNSForwarderZoneStatistics',
        :'queries_answered_locally' => :'Integer',
        :'used_cache_statistics' => :'Array<NsxTPerNodeUsedCacheStatistics>',
        :'configured_cache_size' => :'Integer',
        :'timestamp' => :'Integer',
        :'conditional_forwarder_statistics' => :'Array<NsxTDNSForwarderZoneStatistics>',
        :'total_queries' => :'Integer'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'resource_type')
        self.resource_type = attributes[:'resource_type']
      end

      if attributes.has_key?(:'enforcement_point_path')
        self.enforcement_point_path = attributes[:'enforcement_point_path']
      end

      if attributes.has_key?(:'queries_forwarded')
        self.queries_forwarded = attributes[:'queries_forwarded']
      end

      if attributes.has_key?(:'cached_entries')
        self.cached_entries = attributes[:'cached_entries']
      end

      if attributes.has_key?(:'default_forwarder_statistics')
        self.default_forwarder_statistics = attributes[:'default_forwarder_statistics']
      end

      if attributes.has_key?(:'queries_answered_locally')
        self.queries_answered_locally = attributes[:'queries_answered_locally']
      end

      if attributes.has_key?(:'used_cache_statistics')
        if (value = attributes[:'used_cache_statistics']).is_a?(Array)
          self.used_cache_statistics = value
        end
      end

      if attributes.has_key?(:'configured_cache_size')
        self.configured_cache_size = attributes[:'configured_cache_size']
      end

      if attributes.has_key?(:'timestamp')
        self.timestamp = attributes[:'timestamp']
      end

      if attributes.has_key?(:'conditional_forwarder_statistics')
        if (value = attributes[:'conditional_forwarder_statistics']).is_a?(Array)
          self.conditional_forwarder_statistics = value
        end
      end

      if attributes.has_key?(:'total_queries')
        self.total_queries = attributes[:'total_queries']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if @resource_type.nil?
        invalid_properties.push('invalid value for "resource_type", resource_type cannot be nil.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if @resource_type.nil?
      resource_type_validator = EnumAttributeValidator.new('String', ['NsxTDNSForwarderStatistics'])
      return false unless resource_type_validator.valid?(@resource_type)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] resource_type Object to be assigned
    def resource_type=(resource_type)
      validator = EnumAttributeValidator.new('String', ['NsxTDNSForwarderStatistics'])
      unless validator.valid?(resource_type)
        fail ArgumentError, 'invalid value for "resource_type", must be one of #{validator.allowable_values}.'
      end
      @resource_type = resource_type
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          resource_type == o.resource_type &&
          enforcement_point_path == o.enforcement_point_path &&
          queries_forwarded == o.queries_forwarded &&
          cached_entries == o.cached_entries &&
          default_forwarder_statistics == o.default_forwarder_statistics &&
          queries_answered_locally == o.queries_answered_locally &&
          used_cache_statistics == o.used_cache_statistics &&
          configured_cache_size == o.configured_cache_size &&
          timestamp == o.timestamp &&
          conditional_forwarder_statistics == o.conditional_forwarder_statistics &&
          total_queries == o.total_queries
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [resource_type, enforcement_point_path, queries_forwarded, cached_entries, default_forwarder_statistics, queries_answered_locally, used_cache_statistics, configured_cache_size, timestamp, conditional_forwarder_statistics, total_queries].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = NSXTPolicy.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
